/*******************************************************************************
 * Copyright (c) 2008 Mateusz Matela and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Mateusz Matela - initial API and implementation
 *******************************************************************************/
package org.gsoc.eclipse.tostringgenerator;

import java.util.Map;

import org.gsoc.eclipse.tostringgenerator.internal.UIMessages;

/**
 * <p>
 * Classes implementing this interface provide information about the template
 * determining format of the toString() method output. The interface also
 * provides tokens that must be implemented by every template and their
 * descriptions. Implementing classes may add their own tokens.
 * </p>
 * <p>
 * The template is divided into four parts:
 * <ul>
 * <li><b>Beginning</b> - this part can contain only object related tokens and
 * is used to generate the beginning of the generated <code>String</code>.</li>
 * <li><b>Body</b> - content of this part is used once for every processed
 * member. All member related and object related tokens are available.
 * <li><b>Separator</b></li> - this part does not contain tokens and is used to
 * separate members in the generated <code>String</code>.
 * <li><b>Ending</b> - this part can also contain only object related tokens and
 * it is used to generate the ending of the output <code>string</code>.</li>
 * </ul>
 * </p>
 */
public interface IToStringTemplateParser {
	/** The token that inserts the name of the object's class **/
	public final static String OBJECT_NAME_TOKEN = "${object.className}"; //$NON-NLS-1$

	/** The token that inserts a call to <code>this.getClass.getName()</code> **/
	public final static String OBJECT_GET_NAME_TOKEN = "${object.getClassName}"; //$NON-NLS-1$

	/** The token that inserts a call to <code>super.toString()</code> **/
	public final static String OBJECT_SUPER_TOSTRING_TOKEN = "${object.superToString}"; //$NON-NLS-1$

	/** The token that inserts a call to <code>this.hashCode()</code> **/
	public final static String OBJECT_HASHCODE_TOKEN = "${object.hashCode}"; //$NON-NLS-1$

	/**
	 * The token that inserts a call to
	 * <code>System.identityHashCode(this)</code>
	 **/
	public final static String OBJECT_SYSTEM_HASHCODE_TOKEN = "${object.identityHashCode}"; //$NON-NLS-1$

	/** The token that inserts the member name **/
	public final static String MEMBER_NAME_TOKEN = "${member.name}"; //$NON-NLS-1$

	/**
	 * The token that inserts the member name followed by parenthesis in case of
	 * methods
	 **/
	public final static String MEMBER_NAME_PARENTHESIS_TOKEN = "${member.name()}"; //$NON-NLS-1$

	/** The token that inserts the values of a member **/
	public final static String MEMBER_VALUE_TOKEN = "${member.value}"; //$NON-NLS-1$

	/** The token used for determining separator between members **/
	public final static String OTHER_MEMBERS_TOKEN = "${otherMembers}"; //$NON-NLS-1$

	public final static String[] OBJECT_RELAED_TOKENS = { OBJECT_NAME_TOKEN,
			OBJECT_GET_NAME_TOKEN, OBJECT_SUPER_TOSTRING_TOKEN,
			OBJECT_HASHCODE_TOKEN, OBJECT_SYSTEM_HASHCODE_TOKEN };

	public final static String[] MEMBER_RELATED_TOKENS = { MEMBER_NAME_TOKEN,
			MEMBER_NAME_PARENTHESIS_TOKEN, MEMBER_VALUE_TOKEN };

	public final static String[] OBJECT_AND_MEMBER_RELATED_TOKENS = {
			OBJECT_NAME_TOKEN, OBJECT_GET_NAME_TOKEN,
			OBJECT_SUPER_TOSTRING_TOKEN, OBJECT_HASHCODE_TOKEN,
			OBJECT_SYSTEM_HASHCODE_TOKEN, MEMBER_NAME_TOKEN,
			MEMBER_NAME_PARENTHESIS_TOKEN, MEMBER_VALUE_TOKEN };

	/** A default template for toString() format **/
	public final static String DEFAULT_TEMPLATE = "${object.className} [${member.name()}=${member.value}, ${otherMembers}]"; //$NON-NLS-1$

	/** Tokens that can be used in toSting() format template **/
	public final static String[] tokens = { OBJECT_NAME_TOKEN,
			OBJECT_GET_NAME_TOKEN, OBJECT_SUPER_TOSTRING_TOKEN,
			OBJECT_HASHCODE_TOKEN, OBJECT_SYSTEM_HASHCODE_TOKEN,
			MEMBER_NAME_TOKEN, MEMBER_NAME_PARENTHESIS_TOKEN,
			MEMBER_VALUE_TOKEN, OTHER_MEMBERS_TOKEN };

	/** Descriptions of tokens in toString format template **/
	public final static String[] tokenDescriptions = {
			UIMessages.IToStringTemplateParser_templateDescriptionClassName,
			UIMessages.IToStringTemplateParser_templateDescriptionGetClassName,
			UIMessages.IToStringTemplateParser_templateDescriptionSuperToString,
			UIMessages.IToStringTemplateParser_templateDescriptionHashCode,
			UIMessages.IToStringTemplateParser_templateDescriptionIdentityHashCode,
			UIMessages.IToStringTemplateParser_templateDescriptionMemberName,
			UIMessages.IToStringTemplateParser_templateDescriptionMemberNameWithParenthesis,
			UIMessages.IToStringTemplateParser_templateDescriptionMemberValue,
			UIMessages.IToStringTemplateParser_templateDescriptionOtherFields };

	/**
	 * Returns the tokens that can be used with this parser. Usually the parser
	 * should just return <code>this.tokens</code> unless it supports additional
	 * tokens.
	 * 
	 * @return array of strings representing tokens that can be used in a
	 *         toString template
	 */
	public String[] getTokens();

	/**
	 * Returns a map containing descriptions of the tokens. For every token
	 * contained in an array returned by {@link #getTokens()} the map should
	 * contain a string describing it.
	 * 
	 * @return a map with descriptions for the tokens
	 */
	public Map getTokenDescriptions();

	/**
	 * Parses the template. The results can be retrieved using
	 * {@link #getBeginning()}, {@link #getBody()}, {@link #getEnding()} and
	 * {@link #getSeparator()}.
	 * 
	 * @param template
	 */
	public void parseTemplate(String template);

	/**
	 * Returns elements from the beginning part of the template. Elements can be
	 * plain strings and object related tokens.
	 * 
	 * @return array of strings representing elements of the template
	 */
	public String[] getBeginning();

	/**
	 * Returns elements from the 'body' (main) part of the template. Elements
	 * can be plain strings, member related tokens and object related tokens.
	 * 
	 * @return array of strings representing elements of the template
	 */
	public String[] getBody();

	/**
	 * Returns elements from the ending part of the template. Elements can be
	 * plain strings and object related tokens.
	 * 
	 * @return array of strings representing elements of the template
	 */
	public String[] getEnding();

	/**
	 * 
	 * @return a string representing the separator
	 */
	public String getSeparator();

}
