/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.gsoc.eclipse.tostringgenerator.internal.actions;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.IClassFile;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.corext.util.Messages;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.javaeditor.JavaEditor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.MessageDialogWithToggle;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.osgi.util.TextProcessor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.editors.text.EditorsUI;

/**
 * This is a striped copy of
 * {@link org.eclipse.jdt.internal.ui.actions.ActionUtil}. It is copied here
 * because it's interface has changed significantly between Eclipse versions 3.2
 * and 3.4. This way the same version of the ToSring() Generator plug-in can be
 * run in Eclipse 3.2, 3.3 and 3.4.
 */
public class ActionUtil {

	private ActionUtil() {
	}

	private static boolean isProcessable(JavaEditor editor) {
		if (editor == null)
			return true;
		Shell shell = editor.getSite().getShell();
		IJavaElement input = getEditorInputJavaElement(editor, true);
		// if a Java editor doesn't have an input of type Java element
		// then it is for sure not on the build path
		if (input == null) {
			MessageDialog.openInformation(shell,
					UIMessages.GenerateToStringActionDelegate_cannotPerofm,
					UIMessages.GenerateToStringActionDelegate_notInBuildBath);
			return false;
		}
		return ActionUtil.isProcessable(shell, input);
	}

	private static IJavaElement getEditorInputJavaElement(IEditorPart editor,
			boolean primaryOnly) {
		IEditorInput editorInput = editor.getEditorInput();
		if (editorInput == null)
			return null;

		IJavaElement je = null;// = JavaUI.getEditorInputTypeRoot(editorInput);
		ICompilationUnit cu = JavaPlugin.getDefault().getWorkingCopyManager()
				.getWorkingCopy(editorInput);
		if (cu != null)
			je = cu;

		IJavaElement je2 = (IJavaElement) editorInput
				.getAdapter(IJavaElement.class);
		if (je2 instanceof ICompilationUnit || je2 instanceof IClassFile)
			je = je2;

		if (je != null || primaryOnly)
			return je;

		return JavaPlugin.getDefault().getWorkingCopyManager().getWorkingCopy(
				editorInput, false);
	}

	public static boolean isProcessable(Shell shell, IJavaElement element) {
		if (element == null)
			return true;
		if (isOnBuildPath(element))
			return true;
		MessageDialog.openInformation(shell,
				UIMessages.GenerateToStringActionDelegate_cannotPerofm,
				UIMessages.ActionUtil_errorResourceNotOnBuildPath);
		return false;
	}

	public static boolean isEditable(JavaEditor editor) {
		if (!isProcessable(editor))
			return false;

		return editor.validateEditorInputState();
	}

	private static boolean isEditable(Shell shell, IJavaElement element) {
		if (!ActionUtil.isProcessable(shell, element))
			return false;

		IJavaElement cu = element.getAncestor(IJavaElement.COMPILATION_UNIT);
		if (cu != null) {
			IResource resource = cu.getResource();
			if (resource != null && resource.isDerived()) {

				// see org.eclipse.ui.texteditor.AbstractDecoratedTextEditor#
				// validateEditorInputState()
				final String warnKey = "warn_if_input_derived"; //$NON-NLS-1$
				IPreferenceStore store = EditorsUI.getPreferenceStore();
				if (!store.getBoolean(warnKey))
					return true;

				MessageDialogWithToggle toggleDialog = MessageDialogWithToggle
						.openYesNoQuestion(
								shell,
								UIMessages.GenerateToStringActionDelegate_derivedFileCaption,
								Messages
										.format(
												UIMessages.GenerateToStringActionDelegate_derivedFileQuestion,
												getPathLabel(resource
														.getFullPath(), false)),
								UIMessages.GenerateToStringActionDelegate_derivedFileDontWarn,
								false, null, null);

				EditorsUI.getPreferenceStore().setValue(warnKey,
						!toggleDialog.getToggleState());

				return toggleDialog.getReturnCode() == IDialogConstants.YES_ID;
			}
		}
		return true;
	}

	/**
	 * Returns the label of a path.
	 * 
	 * @param path
	 *            the path
	 * @param isOSPath
	 *            if <code>true</code>, the path represents an OS path, if
	 *            <code>false</code> it is a workspace path.
	 * @return the label of the path to be used in the UI.
	 */
	private static String getPathLabel(IPath path, boolean isOSPath) {
		String label;
		if (isOSPath) {
			label = path.toOSString();
		} else {
			label = path.makeRelative().toString();
		}
		return TextProcessor.process(label);
	}

	public static boolean isOnBuildPath(IJavaElement element) {
		// fix for bug http://dev.eclipse.org/bugs/show_bug.cgi?id=20051
		if (element.getElementType() == IJavaElement.JAVA_PROJECT)
			return true;
		IJavaProject project = element.getJavaProject();
		try {
			if (!project.isOnClasspath(element))
				return false;
			IProject resourceProject = project.getProject();
			if (resourceProject == null)
				return false;
			IProjectNature nature = resourceProject
					.getNature(JavaCore.NATURE_ID);
			// We have a Java project
			if (nature != null)
				return true;
		} catch (CoreException e) {
		}
		return false;
	}

	/**
	 * Check whether <code>editor</code> and <code>element</code> are
	 * processable and editable. If the editor edits the element, the validation
	 * is only performed once. If necessary, ask the user whether the file(s)
	 * should be edited.
	 * 
	 * @param editor
	 *            an editor, or <code>null</code> iff the action was not
	 *            executed from an editor
	 * @param shell
	 *            a shell to serve as parent for a dialog
	 * @param element
	 *            the element to check, cannot be <code>null</code>
	 * @return <code>true</code> if the element can be edited,
	 *         <code>false</code> otherwise
	 */
	public static boolean isEditable(JavaEditor editor, Shell shell,
			IJavaElement element) {
		if (editor != null) {
			IJavaElement input = getEditorInputJavaElement(editor, true);
			if (input != null
					&& input.equals(element
							.getAncestor(IJavaElement.COMPILATION_UNIT)))
				return isEditable(editor);
			return isEditable(editor) && isEditable(shell, element);
		}
		return isEditable(shell, element);
	}

}
